﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management.Automation;
using Microsoft.SharePoint.PowerShell;
using Microsoft.SharePoint.Administration;
using System.Net;
using DevLeap.SP2010.ProtocolServiceAppProxy;

namespace DevLeap.SP2010.ProtocolServiceApp.PowerShellRegistration
{
    [Cmdlet(VerbsCommon.New, "NewProtocolServiceApplication",
        DefaultParameterSetName = NewProtocolServiceApplication.DefaultParameterSetName,
        SupportsShouldProcess = true)]
    [SPCmdlet(RequireLocalFarmExist = true, RequireUserFarmAdmin = true)]
    internal sealed class NewProtocolServiceApplication : SPCmdlet
    {
        private const string DefaultParameterSetName = "Default";
        private const string DatabaseServerParameterSetName = "DatabaseServer";
        private const string DefaultDatabaseName = "ProtocolService";

        [Parameter(Mandatory = true, Position = 0)]
        [ValidateNotNullOrEmpty]
        public string Name { get; set; }

        [Parameter(Mandatory = true)]
        [ValidateNotNull]
        public SPIisWebServiceApplicationPoolPipeBind ApplicationPool { get; set; }

        [Parameter(Mandatory = false)]
        [ValidateNotNullOrEmpty]
        public string DatabaseName { get; set; }

        [Parameter(ParameterSetName = DatabaseServerParameterSetName, Mandatory = true)]
        [ValidateNotNullOrEmpty]
        public string DatabaseServer { get; set; }

        [Parameter(ParameterSetName = DatabaseServerParameterSetName, Mandatory = false)]
        [ValidateNotNullOrEmpty]
        public string FailoverDatabaseServer { get; set; }

        [Parameter(ParameterSetName = DatabaseServerParameterSetName, Mandatory = false)]
        [ValidateNotNull]
        public PSCredential DatabaseCredentials { get; set; }

        protected override void InternalProcessRecord()
        {
            // Ensure that the farm level required objects exist
            SPFarm farm = SPFarm.Local;
            if (null == farm)
            {
                ThrowTerminatingError(new InvalidOperationException("SharePoint server farm not found."),
                    ErrorCategory.ResourceUnavailable, this);
            }

            // Ensure that the sample web service exists
            ProtocolService service = farm.Services.GetValue<ProtocolService>();
            if (service == null)
            {
                ThrowTerminatingError(new InvalidOperationException("Protocol Service not found."),
                    ErrorCategory.ResourceUnavailable, this);
            }

            // Check for an existing sample web service application
            ProtocolServiceApplication existingServiceApplication = service.Applications.GetValue<ProtocolServiceApplication>();
            if (existingServiceApplication != null)
            {
                WriteError(new InvalidOperationException("Protocol Service application exists."),
                    ErrorCategory.ResourceExists, existingServiceApplication);

                // Skip the current record
                SkipProcessCurrentRecord();
            }

            // Get a reference to the specified application pool
            SPIisWebServiceApplicationPool applicationPool = this.ApplicationPool.Read();
            if (null == applicationPool)
            {
                WriteError(new InvalidOperationException("The specified application pool could not be found."),
                    ErrorCategory.InvalidArgument, this);

                // Skip the current record.
                SkipProcessCurrentRecord();
            }

            if (String.IsNullOrEmpty(this.DatabaseName))
                this.DatabaseName = NewProtocolServiceApplication.DefaultDatabaseName;

            // Construct parameters for new database
            SPDatabaseParameters databaseParameters = SPDatabaseParameters.CreateParameters(
                this.DatabaseName,
                this.DatabaseServer,
                (null == this.DatabaseCredentials ? null : this.DatabaseCredentials.GetNetworkCredential()),
                this.FailoverDatabaseServer,
                SPDatabaseParameterOptions.ResolveNameConflict);

            if (ShouldProcess(this.Name))
            {
                // Create the sample application
                ProtocolServiceApplication serviceApplication = ProtocolServiceApplication.Create(
                    this.Name, service, applicationPool, databaseParameters);

                if (serviceApplication == null)
                {
                    WriteError(new NullReferenceException("ProtocolServiceApplication"),
                        ErrorCategory.InvalidResult, this);
                }

                // Provision the sample application
                serviceApplication.Provision();

                // Write the new sample application to the pipeline
                WriteObject(serviceApplication);
            }
        }
    }
}
