﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Utilities;
using System.IO;
using System.ServiceModel;
using Microsoft.SharePoint.Administration.AccessControl;
using DevLeap.SP2010.ProtocolServiceAppProxy;
using DevLeap.SP2010.ProtocolServiceAppContracts;
using Microsoft.SharePoint;

namespace DevLeap.SP2010.ProtocolServiceApp
{
    [IisWebServiceApplicationBackupBehavior]
    [ServiceBehavior(
        InstanceContextMode = InstanceContextMode.PerSession,
        ConcurrencyMode = ConcurrencyMode.Multiple,
#if DEBUG
        IncludeExceptionDetailInFaults = true
#endif
        )]
    [System.Runtime.InteropServices.Guid("151EA861-1B53-4f2e-A714-1B500FAE9C6E")]
    public class ProtocolServiceApplication : SPIisWebServiceApplication, IProtocolService
    {
        [Persisted]
        private ProtocolServiceDatabase _database;

        #region Constructors

        // Default constructor for serialization purposes
        public ProtocolServiceApplication()
        { }

        internal ProtocolServiceApplication(string name, ProtocolService service, SPIisWebServiceApplicationPool applicationPool, ProtocolServiceDatabase database)
            : base(name, service, applicationPool)
        {
            if (database == null)
            {
                throw new ArgumentNullException("database");
            }
            this._database = database;
        }

        #endregion

        #region Factory methods

        public static ProtocolServiceApplication Create(string name, ProtocolService service, SPIisWebServiceApplicationPool applicationPool, SPDatabaseParameters databaseParameters)
        {
            // Parameters validation
            if (name == null)
                throw new ArgumentNullException("name");
            if (service == null)
                throw new ArgumentNullException("service");
            if (applicationPool == null)
                throw new ArgumentNullException("applicationPool");
            if (databaseParameters == null)
                throw new ArgumentNullException("databaseParameters");

            // Register the database
            ProtocolServiceDatabase database = new ProtocolServiceDatabase(databaseParameters);
            database.Update();

            // Create and persist the service application
            ProtocolServiceApplication serviceApplication = new ProtocolServiceApplication(
                name,
                service,
                applicationPool,
                database);
            serviceApplication.Update();

            // Register endpoints
            serviceApplication.AddServiceEndpoint("http", SPIisWebServiceBindingType.Http);
            serviceApplication.AddServiceEndpoint("https", SPIisWebServiceBindingType.Https, "secure");
            // serviceApplication.AddServiceEndpoint("tcp", SPIisWebServiceBindingType.NetTcp);
            // serviceApplication.AddServiceEndpoint("pipe", SPIisWebServiceBindingType.NetPipe);

            return (serviceApplication);
        }

        #endregion

        #region SPIisWebServiceApplication

        protected override string DefaultEndpointName
        {
            get { return ("http"); }
        }

        public override string TypeName
        {
            get { return ("Protocol Service Application"); }
        }

        protected override string InstallPath
        {
            get { return Path.GetFullPath(SPUtility.GetGenericSetupPath(@"WebServices\ProtocolService")); }
        }

        protected override string VirtualPath
        {
            get
            {
                return "ProtocolService.svc";
            }
        }

        public override Guid ApplicationClassId
        {
            get { return (new Guid("C7E904DA-9DF0-4038-9806-025EEA58C437")); }
        }

        public override Version ApplicationVersion
        {
            get { return (new Version("1.0.0.0")); }
        }

        public override void Provision()
        {
            base.Status = SPObjectStatus.Provisioning;
            this.Update();

            this._database.Provision();
            base.Provision();

            //base.Status = SPObjectStatus.Online;
            //this.Update();
        }

        public override void Unprovision(bool deleteData)
        {
            base.Status = SPObjectStatus.Unprovisioning;
            this.Update();

            base.Unprovision(deleteData);
            if (deleteData && (this._database != null))
            {
                this._database.Unprovision();
            }

            base.Status = SPObjectStatus.Disabled;
            this.Update();
        }

        protected override void OnProcessIdentityChanged(System.Security.Principal.SecurityIdentifier processSecurityIdentifier)
        {
            base.OnProcessIdentityChanged(processSecurityIdentifier);
            _database.GrantApplicationPoolAccess(processSecurityIdentifier);
        }

        public override SPAdministrationLink ManageLink
        {
            get
            {
                return new SPAdministrationLink(String.Format("/_admin/ProtocolService/ProtocolServiceManage.aspx?appid=", this.Id));
            }
        }

        public override SPAdministrationLink PropertiesLink
        {
            get
            {
                return new SPAdministrationLink(String.Format("/_admin/ProtocolService/ProtocolServiceProperties.aspx?appid=", this.Id));
            }
        }

        protected override SPNamedCentralAdministrationRights[] AdministrationAccessRights
        {
            get
            {
                return base.AdministrationAccessRights;
            }
        }

        protected override SPNamedIisWebServiceApplicationRights[] AccessRights
        {
            get
            {
                return new SPNamedIisWebServiceApplicationRights[]
                {
                    SPNamedIisWebServiceApplicationRights.FullControl,
                    new SPNamedIisWebServiceApplicationRights("Request", ProtocolServiceAccessRights.Request),
                    SPNamedIisWebServiceApplicationRights.Read,
                };
            }
        }

        #endregion

        #region IProtocolService

        [OperationBehavior(Impersonation = ImpersonationOption.Allowed)]
        public string GenerateProtocolNumber(string contentUri)
        {
            DemandAccess(ProtocolServiceAccessRights.Request);

            return (String.Format("{0}-{1}",
                Guid.NewGuid(),
                contentUri));
        }

        #endregion

        #region Internal methods

        internal static ProtocolServiceApplication GetLocalApplicationByName(string name)
        {
            return ProtocolService.CurrentInstance.GetWebApplicationByName(name);
        }

        #endregion
    }
}
