﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint.Administration;
using System.ServiceModel;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using System.Configuration;
using System.ServiceModel.Configuration;
using System.ServiceModel.Channels;
using DevLeap.SP2010.ProtocolServiceAppContracts;

namespace DevLeap.SP2010.ProtocolServiceAppProxy
{
    [IisWebServiceApplicationProxyBackupBehavior]
    [System.Runtime.InteropServices.Guid("B5EEF05C-5C48-471c-A1F4-21B5726DA827")]
    public sealed class ProtocolServiceApplicationProxy: SPIisWebServiceApplicationProxy
    {
        private ChannelFactory<IProtocolService> _channelFactory;
        private Object _channelFactorySyncLock = new Object();
        private String _endpointConfigurationName;

        [Persisted]
        private SPServiceLoadBalancer _loadBalancer;

        #region Constructors

        // An empty public constructor required for serialization.
        public ProtocolServiceApplicationProxy()
            : base()
        {
        }

        public ProtocolServiceApplicationProxy(string name, SPIisWebServiceProxy serviceProxy, Uri serviceEndpointUri)
            : base(name, serviceProxy, serviceEndpointUri)
        {
            // Create a new round-robin load balancer
            _loadBalancer = new SPRoundRobinServiceLoadBalancer(serviceEndpointUri);
        }

        #endregion

        #region SPIisWebServiceApplicationProxy

        public override string TypeName
        {
            get { return ("Protocol Service Application Proxy"); }
        }

        public override void Provision()
        {
            // Provision the load balancer
            _loadBalancer.Provision();

            base.Provision();

            // Ensure the provisioned load balancer gets persisted back to the database  
            this.Update();
        }

        public override void Unprovision(bool deleteData)
        {
            // Unprovision the load balancer
            _loadBalancer.Unprovision();

            base.Unprovision(deleteData);

            // Ensure the un-provisioned load balancer gets persisted back to the database  
            this.Update();
        }

        #endregion

        #region WCF code plumbing

        private ChannelFactory<T> CreateChannelFactory<T>(string endpointConfigurationName)
        {
            String clientConfigPath = SPUtility.GetGenericSetupPath(@"WebClients\ProtocolService");
            Configuration clientConfig = OpenClientConfiguration(clientConfigPath);
            ConfigurationChannelFactory<T> factory = new ConfigurationChannelFactory<T>(endpointConfigurationName, clientConfig, null);

            // Configure the channel factory for claims authentication
            factory.ConfigureCredentials(SPServiceAuthenticationMode.Claims);

            return (factory);
        }

        internal delegate void CodeToRunOnApplicationProxy(ProtocolServiceApplicationProxy applicationProxy);

        internal static void Invoke(SPServiceContext serviceContext, CodeToRunOnApplicationProxy codeBlock)
        {
            if (serviceContext == null)
                throw new ArgumentNullException("serviceContext");

            ProtocolServiceApplicationProxy proxy =
                (ProtocolServiceApplicationProxy)serviceContext.GetDefaultProxy(typeof(ProtocolServiceApplicationProxy));

            if (proxy == null)
                throw new InvalidOperationException("Protocol Service Application proxy not found!");

            // Ensure that SPServiceContext.Current is set correctly.
            using (new SPServiceContextScope(serviceContext))
            {
                // Execute the delegate
                codeBlock(proxy);
            }
        }

        private delegate void CodeToRunOnChannel(IProtocolService serviceContract);

        private void ExecuteOnChannel(String operationName, ExecuteOptions options, CodeToRunOnChannel codeBlock)
        {
            using (new SPMonitoredScope("ExecuteOnChannel:" + operationName))
            {
                SPServiceLoadBalancerContext loadBalancerContext = _loadBalancer.BeginOperation();
                try
                {
                    // Get a channel to the service application endpoint
                    IChannel channel = (IChannel)GetChannel(loadBalancerContext.EndpointAddress, options);
                    try
                    {
                        // Execute the code
                        codeBlock((IProtocolService)channel);
                        channel.Close();
                    }
                    catch (TimeoutException)
                    {
                        loadBalancerContext.Status = SPServiceLoadBalancerStatus.Failed;
                        throw;
                    }
                    catch (EndpointNotFoundException)
                    {
                        loadBalancerContext.Status = SPServiceLoadBalancerStatus.Failed;
                        throw;
                    }
                    finally
                    {
                        if (channel.State != CommunicationState.Closed)
                            channel.Abort();
                    }
                }
                finally
                {
                    loadBalancerContext.EndOperation();
                }
            }
        }

        private string GetEndpointConfigurationName(Uri address)
        {
            if (address == null)
                throw new ArgumentNullException("address");

            string configName;

            if (address.Scheme == Uri.UriSchemeHttp)
            {
                configName = "http";
            }
            else if (address.Scheme == Uri.UriSchemeHttps)
            {
                configName = "https";
            }
            //else if (address.Scheme == Uri.UriSchemeNetTcp)
            //{
            //    configName = "tcp";
            //}
            //else if (address.Scheme == Uri.UriSchemeNetPipe)
            //{
            //    configName = "pipe";
            //}
            else
            {
                throw new NotSupportedException("Unsupported endpoint address");
            }

            return (configName);
        }

        private IProtocolService GetChannel(Uri address, ExecuteOptions options)
        { 
            if (address == null)
                throw new ArgumentNullException("address");

            string endpointConfigName = GetEndpointConfigurationName(address);

            // Singleton to get the ChannelFactory instance
            if ((_channelFactory == null) || (endpointConfigName != _endpointConfigurationName))
            {
                lock (_channelFactorySyncLock)
                {
                    if ((_channelFactory == null) || (endpointConfigName != _endpointConfigurationName))
                    {
                        _channelFactory = CreateChannelFactory<IProtocolService>(endpointConfigName);
                        _endpointConfigurationName = endpointConfigName;
                    }
                }
            }

            IProtocolService channel;

            if (ExecuteOptions.AsProcess == (options & ExecuteOptions.AsProcess))
            {
                // Create a channel that acts as the process when authenticating with the service
                channel = _channelFactory.CreateChannelAsProcess<IProtocolService>(new EndpointAddress(address));
            }
            else
            {
                // Create a channel that acts as the logged on user when authenticating with the service
                channel = _channelFactory.CreateChannelActingAsLoggedOnUser<IProtocolService>(new EndpointAddress(address));
            }

            return (channel);
        }

        #endregion

        #region IProtocolService

        public String GenerateProtocolNumber(String contentUri, ExecuteOptions options)
        { 
            String result = null;

            ExecuteOnChannel("GenerateProtocolNumber", options, channel => result = channel.GenerateProtocolNumber(contentUri));

            return (result);
        }

        #endregion

        #region Custom implementation

        public void AddToDefaultGroup()
        {
            SPServiceApplicationProxyGroup group = SPServiceApplicationProxyGroup.Default;
            group.Add(this);
            group.Update();
        }
        
        #endregion
    }
}
